@@ -3,7 +3,7 @@
 # Bidirectional map
 
 
-class BiDict:
+class BiDict(object):
     def __init__(self):
         self.forward = {}
         self.backward = {}
